
/*
 *	This test measures the MPI point-to-point latency.
 *	It must be run with exactly 2 processes.  The test runs on T3E and 
 *	IRIX systems.
 *
 *	kaf 1/11/99 - added test.  It shows 14 usec latency on T3E and
 *		      15 usec latency on O2K.
 */
#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>
#ifndef _CRAY
#include "rtc.h"
#endif

#define SZ 1
long trg[SZ], src[SZ];

main(int argc, char **argv)
{
  int itry;
  MPI_Comm comm;
  MPI_Group group_world;
  int me,comm_size, other;
  MPI_Status status;
  long long t0, t1;
  long bytes = SZ * sizeof(long);

  MPI_Init(&argc,&argv);

  comm = MPI_COMM_WORLD;

  MPI_Comm_group(comm,&group_world);
  MPI_Comm_rank(comm, &me);
  MPI_Comm_size(comm,&comm_size);


  other = ! me;

  if (me == 0) {
	printf("%s system:  clock rate %ld Hz.  Clock period is %f microseconds\n\n",
#if __mips
		"MIPS"
#else
		"Linux"
#endif
		, (long)IRTC_RATE(), (double)(1.0/(float)IRTC_RATE())*1000000.0);
  }

/*
 * Measure ping-pong latency
 */
#define TRIES 1000

  shmem_barrier_all();
  t0 = _rtc();
  for(itry=0; itry < TRIES; itry ++) {
      if(me == 0) {
        MPI_Send(src, SZ, MPI_LONG, other, 1001, MPI_COMM_WORLD);
        MPI_Recv(trg, SZ, MPI_LONG, other, 1001, MPI_COMM_WORLD, &status);
      } else if(me == 1) {
        MPI_Recv(trg, SZ, MPI_LONG, other, 1001, MPI_COMM_WORLD, &status);
        MPI_Send(src, SZ, MPI_LONG, other, 1001, MPI_COMM_WORLD);
      }
  }
  t1 = _rtc();

   /* sender measures */
   if (me == 0) {
	double clocks;
        double usec;

	clocks = t1-t0;
	clocks /= TRIES;	
	clocks /= 2;	/* half of ping-pong */

        usec = ( (clocks) / (double)IRTC_RATE() ) * 1000000.0;

        printf("ping-pong %ld bytes xfered in %ld clocks,  %f microseconds\n",
                bytes, (long)clocks, usec);
   }


  MPI_Finalize(); 
  return 0;
}
