/* USMID %Z%%M%	%I%	%G% %U% */
/*
 *	(C) COPYRIGHT CRAY RESEARCH, INC.
 *	UNPUBLISHED PROPRIETARY INFORMATION.
 *	ALL RIGHTS RESERVED.
 */

/*****************************************************************************
 *
 * Exported interfaces
 *
 *
 *  long long _rtc(void)
 *
 *	Returns the current clock value.
 *
 *  double setup_rtc(void);
 *
 *       Returns the number of microseconds per clock period.
 *
 *   long long IRTC_RATE(void)
 *
 *	 Returns the number of clock periods per second.
 *
 ****************************************************************************/



#ifdef __mips
/*
 * The 7.3 compiler expects _rtc() to be long, not long long.  But this
 * definition is for long long because we need a 64 bit value.
 */
#define       _rtc    _rtc_longlong


typedef	long long	rtc_t;
extern rtc_t _rtc(void);

#else	/* CRAY */

#include <unistd.h>

extern long long IRTC_RATE(void);
typedef	long		rtc_t;
static double
setup_rtc(void)
{ 
        return 1000000.0 / (double) sysconf(_SC_CLK_TCK);
}
#endif


#ifdef __mips



extern int _rtc_initialized;
static unsigned int _cycleval;                 /* picoseconds per tick */


#include <errno.h>
#include <stddef.h>
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/syssgi.h>
/* #include <mpp/shmem.h> */


#if CYCLE_COUNTER_IS_64BIT
typedef unsigned long long iotimer_t;
#else
typedef unsigned int iotimer_t;
#endif


unsigned int _cycleval;			/* picoseconds per tick */

volatile unsigned int  *_clockaddr_low, *_clockaddr_high;

static unsigned int zero = 0;
static unsigned int one = 1;
int _rtc_setup_called;


/*
 *	setup_rtc
 *
 *	Initializes real time clock.
 *
 * 	Return value: the number of microseconds per clock period.
 */
static double
setup_rtc(void)
{
  __psunsigned_t phys_addr, raddr;
  int fd, poffmask;
  int cycsz;
  void *clockaddr;
 
  if (_rtc_setup_called == 1) goto signoff;
  _rtc_setup_called = 1;

  poffmask = getpagesize() - 1;
  phys_addr = syssgi(SGI_QUERY_CYCLECNTR, &_cycleval);
  cycsz = syssgi(SGI_CYCLECNTR_SIZE);
  raddr = phys_addr & ~poffmask;
  fd = open("/dev/mmem", O_RDONLY);
  clockaddr = mmap(0, poffmask, PROT_READ, MAP_PRIVATE, fd, (off_t)raddr);
  if ((void*)clockaddr == MAP_FAILED) {
	fprintf(stderr, "error %d on _rtc() setup, kludging it\n",errno);
	clockaddr = &one;
  }
  else {
        clockaddr = (void *)((__psunsigned_t)clockaddr +
                            (phys_addr & poffmask));
  }
  if (cycsz == 64) {
	_clockaddr_high = clockaddr;
	_clockaddr_low  = (unsigned int*)clockaddr + 1;
  }
  else if (cycsz == 32) {
	_clockaddr_high = &zero;
	_clockaddr_low = clockaddr;
  }
signoff:
  return (double)_cycleval /1000000.0;	/* picosec to microsec */
}

/*
 *  IRTC_RATE
 *
 *	Returns hertz rate of clock.
 */
long long
IRTC_RATE()
{
	return (long long) ( (1.0/setup_rtc()) * 1e+6);
}


static rtc_t
_rtc()  
{
  if (_rtc_setup_called == 0)
	setup_rtc();
  return ((unsigned long long)(*_clockaddr_high) << 32ULL) + *_clockaddr_low;
}


/*
 *      Fortran callable
 */
long long
irtc_()
{
        return _rtc();
}



/*
 *	get_clocks_per_sec
 *
 *	Return the integer number of clocks per second (HZ).
 */
long
get_clocks_per_sec(void)
{
        return (long)(1000000000000.0 / (float)_cycleval);
}

#endif	/* __mips */

